/*
 * IPSECv6 code
 * Copyright (C) 2000 Stefan Schlott
 * Copyright (C) 2001 Florian Heissenhuber
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.  See <http://www.fsf.org/copyleft/gpl.txt>.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * RCSID $Id: algos-core.c,v 1.1.1.1 2001/05/22 06:14:04 miyazawa Exp $
 */
#include <linux/config.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <asm/types.h>
#include <linux/stddef.h>
#include <linux/types.h>
#include <linux/proc_fs.h>
#include <linux/spinlock.h>
#include <net/ipsec6_algos.h>
#include "algos.h"
#include "hash-dummy.h"
#include "hash-hmac-md5.h"
#include "enc-null.h"
#include "enc-cbcdes.h"
#include "libdes/des.h"
#include "enc-cbc3des.h"

#define IPSEC6_ALGO_ENC "ipsec6-algo-enc"
#define IPSEC6_ALGO_HASH "ipsec6-algo-hash"

EXPORT_SYMBOL(ipsec_register_enc);
EXPORT_SYMBOL(ipsec_unregister_enc);
EXPORT_SYMBOL(ipsec_register_hash);
EXPORT_SYMBOL(ipsec_unregister_hash);


/* To increase performance, use arrays with function pointers instead
   of linked lists. */

struct ipsec_enc_t* ipsec_enc[256];
rwlock_t ipsec_enc_lock = RW_LOCK_UNLOCKED;

struct ipsec_hash_t* ipsec_hash[256];
rwlock_t ipsec_hash_lock = RW_LOCK_UNLOCKED;



int ipsec_register_enc(u_int8_t no, char* name, 
	int key_size, int iv_size, int block_size,
	encrypt_data_t *encrypt_data, encrypt_data_t *decrypt_data) 
{
	struct ipsec_enc_t *new;
	
	if (ipsec_enc[no]) return 0;
	/* Create new structure */
	new=kmalloc(sizeof(struct ipsec_enc_t),GFP_KERNEL);
	strcpy(new->name,name);
	new->key_size=key_size;
	new->iv_size=iv_size;
	new->block_size=block_size;
	new->encrypt_data=encrypt_data;
	new->decrypt_data=decrypt_data;
	/* Insert into list */
	write_lock(&ipsec_enc_lock);
	ipsec_enc[no]=new;
	write_unlock(&ipsec_enc_lock);
	return 1;
}


struct ipsec_enc_t* ipsec_get_enc(int no) 
{
	struct ipsec_enc_t *result;
	
	read_lock(&ipsec_enc_lock);
	result = ipsec_enc[no];
	read_unlock(&ipsec_enc_lock);
	return result;
}


void ipsec_unregister_enc(u_int8_t no) 
{
	if (!ipsec_enc[no]) return;
	write_lock(&ipsec_enc_lock);
	kfree(ipsec_enc[no]);
	ipsec_enc[no]=NULL;
	write_unlock(&ipsec_enc_lock);
}


int ipsec_register_hash(u_int8_t no, char* name, 
	int key_size, int hash_size,
	calc_hash_t *calc_hash) 
{
	struct ipsec_hash_t *new;
	
	if (ipsec_hash[no]) return 0;
	if (!calc_hash) return 0;
	/* Create new structure */
	new=kmalloc(sizeof(struct ipsec_hash_t),GFP_KERNEL);
	strcpy(new->name,name);
	new->key_size=key_size;
	new->hash_size=hash_size;
	new->calc_hash=calc_hash;
	/* Insert into list */
	write_lock(&ipsec_hash_lock);
	ipsec_hash[no]=new;
	write_unlock(&ipsec_hash_lock);
	return 1;
}


struct ipsec_hash_t* ipsec_get_hash(int no) 
{
	struct ipsec_hash_t *result;
	
	read_lock(&ipsec_hash_lock);
	result = ipsec_hash[no];
	read_unlock(&ipsec_hash_lock);
	return result;
}


void ipsec_unregister_hash(u_int8_t no) 
{
	if (!ipsec_hash[no]) return;
	write_lock(&ipsec_hash_lock);
	kfree(ipsec_hash[no]);
	ipsec_hash[no]=NULL;
	write_unlock(&ipsec_hash_lock);
}


/* See /usr/src/linux/fs/proc/generic.c for infos on /proc */

static int ipsec6_enc_proc_info(char *buffer, char **start, off_t offset, int length)
{
	char *data;
	int datalen;
	int i,cnt;

	/* Build string */
	cnt=0;
	for (i=0; i<=255; i++)
		if (ipsec_enc[i]) cnt++;
	data=kmalloc(cnt*(4+IPSEC_NAME_LEN+5+1)+1,GFP_KERNEL);
	data[0]=0;
	datalen=0;
	for (i=0; i<=255; i++)
		if (ipsec_enc[i]) {
			sprintf(&data[datalen],"%i:%s:%i\n",
				i,ipsec_enc[i]->name,ipsec_enc[i]->key_size);
			datalen=strlen(data);
		}
	data[datalen]='\n';
	datalen++;
	data[datalen]=0;

	/* Return data */	
	if (datalen-offset>length)
		cnt=length; else cnt=datalen-offset;
	if (cnt<0) cnt=0;
	if (cnt>0) memcpy(buffer,&data[offset],cnt);

	/* Free memory */
	kfree(data);	
	return cnt;
}


static int ipsec6_hash_proc_info(char *buffer, char **start, off_t offset, int length)
{
	char *data;
	int datalen;
	int i,cnt;

	/* Build string */
	cnt=0;
	for (i=0; i<=255; i++)
		if (ipsec_hash[i]) cnt++;
	data=kmalloc(cnt*(4+IPSEC_NAME_LEN+5+1)+1,GFP_KERNEL);
	data[0]=0;
	datalen=0;
	for (i=0; i<=255; i++)
		if (ipsec_hash[i]) {
			sprintf(&data[datalen],"%i:%s:%i\n",
				i,ipsec_hash[i]->name,ipsec_hash[i]->key_size);
			datalen=strlen(data);
		}
	data[datalen]='\n';
	datalen++;
	data[datalen]=0;

	/* Return data */	
	if (datalen-offset>length)
		cnt=length; else cnt=datalen-offset;
	if (cnt<0) cnt=0;
	if (cnt>0) memcpy(buffer,&data[offset],cnt);

	/* Free memory */
	kfree(data);	
	return cnt;
}

int algos_init (void)
{
	struct proc_dir_entry *proc_hash, *proc_enc;

	/* Set all tables to NULL */
	memset(ipsec_enc,0,sizeof(ipsec_enc));
	memset(ipsec_hash,0,sizeof(ipsec_hash));
	/* Register dummy algorithms */
	ipsec_register_hash(255,"DUMMY",0,8,hash_dummy_calc_hash);
	ipsec_register_hash(2,"HMAC-MD5",16,12,hash_hmacmd5_calc_hash);
	/*ipsec_register_hash(3,"HMAC-SHA",16,NULL); */
/*FH FreeSWAN doesn't support DES */	
//	ipsec_register_enc(2,"CBC-DES",8,8,8,enc_cbcdes_encrypt_data,enc_cbcdes_decrypt_data);
/*FH end */	
#ifdef CONFIG_IPSEC_ENC_3DES	
	ipsec_register_enc(3,"CBC-3DES",24,8,8,enc_cbc3des_encrypt_data,enc_cbc3des_decrypt_data);
#endif	//CONFIG_IPSEC_ENC_3DES
/*FH FreeSWAN doesn't support NULL */	
//	ipsec_register_enc(11,"NULL",0,0,1,enc_null_encrypt_data,enc_null_decrypt_data);
/*FH end */		
	/* Register proc devices */
	proc_enc = proc_net_create(IPSEC6_ALGO_ENC, S_IFREG | S_IRUSR | S_IRGRP | S_IROTH,ipsec6_enc_proc_info);
	if (!proc_enc) {
		printk (KERN_ERR "Cannot init /proc/net/%s\n",IPSEC6_ALGO_ENC);
		return -ENOMEM;
	}
	proc_hash = proc_net_create(IPSEC6_ALGO_HASH, S_IFREG | S_IRUSR | S_IRGRP | S_IROTH,ipsec6_hash_proc_info);
	if (!proc_hash) {
		printk (KERN_ERR "Cannot init /proc/net/%s\n",IPSEC6_ALGO_HASH);
		proc_net_remove(IPSEC6_ALGO_ENC);
		return -ENOMEM;
	}
	return 0;
}


void algos_cleanup (void)
{
	int i;
	
	/* Unregister proc devices */
	proc_net_remove(IPSEC6_ALGO_ENC);
	proc_net_remove(IPSEC6_ALGO_HASH);
	/* Clean memory */
	write_lock(&ipsec_enc_lock);
	for (i=0; i<=255; i++)
		if (ipsec_enc[i]) kfree(ipsec_enc[i]);
	write_unlock(&ipsec_enc_lock);
	write_lock(&ipsec_hash_lock);
	for (i=0; i<=255; i++)
		if (ipsec_hash[i]) kfree(ipsec_hash[i]);
	write_unlock(&ipsec_hash_lock);
	/* That's it */
	#ifdef ALGOSMOD
	printk(KERN_DEBUG "ipsec6-algos module unregistered\n");
	#endif
}
