/*
 * Copyright (C) 2001 USAGI/WIDE Project.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the project nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * sadb_utils.c include utility functions for SADB handling.
 */

#define __NO_VERSION__
#include <linux/module.h>
#include <linux/version.h>
#ifdef MODVERSIONS
#include <linux/modversions.h>
#endif
#include <linux/version.h>
#include <linux/types.h>
#include <linux/kernel.h>

#include <net/pfkeyv2.h>
#include <net/sadb.h>
#include "sadb_utils.h"

#define BUFSIZE 64
#if 0
int
compare_sockaddr(struct sockaddr* addr1, struct sockaddr* addr2)
{
	int retval = 1;



	/* addr1 == NULL or addr2 == NULL */
	if (!(addr1 && addr2))
		return 1;

	/* Address family is differ from the other. */
	/* This ignores IPv4 mapped address and IPv4 compatible address */
	if (addr1->sa_family != addr2->sa_family)
		return 1;

	switch (addr1->sa_family) {

	case AF_INET:
		return memcmp(addr1, addr2, sizeof(struct sockaddr_in));

	case AF_INET6:
		return memcmp(addr1, addr2, sizeof(struct sockaddr_in6));

	default:
		return 1;
	}
}
#endif

int
compare_address(struct sockaddr* addr1, struct sockaddr* addr2)
{

	/* addr1 == NULL or addr2 == NULL */
	if (!(addr1 && addr2))
		return -EINVAL;

	/* Address family is differ from the other. */
	/* This ignores IPv4 mapped address and IPv4 compatible address */
	if (addr1->sa_family != addr2->sa_family)
		return 1;

	switch (addr1->sa_family) {

	case AF_INET:
		if (((struct sockaddr_in*)addr1)->sin_addr.s_addr 
			== ((struct sockaddr_in*)addr2)->sin_addr.s_addr)
			return 0;
		else
			return 1;

	case AF_INET6:
		return memcmp(((struct sockaddr_in6*)addr1)->sin6_addr.s6_addr,
			 ((struct sockaddr_in6*)addr2)->sin6_addr.s6_addr,
				 sizeof(struct in6_addr));

	default:	
		return 1;
	}
}

char* authtoa(__u8 auth)
{
	switch (auth) {
	case SADB_AALG_MD5HMAC:
		return "md5-hmac";
	case SADB_AALG_SHA1HMAC:
		return "sha1-hmac";
	case SADB_AALG_NONE:
	default:
		return "none";
	}
}

char* esptoa(__u8 esp)
{
	switch (esp) {
	case SADB_EALG_DESCBC:
		return "des-cbc";
	case SADB_EALG_3DESCBC:
		return "3des-cbc";
	case SADB_EALG_NULL:
		return "null";
	case SADB_EALG_NONE:
	default:
		return "none";
	}
}

void dump_sa_lifetime(struct sa_lifetime *lifetime)
{
	if (!lifetime) {
		printk(KERN_INFO "lifetime is null\n");
		return;
	}

	printk(KERN_INFO "bytes=%lu addtime=%lu, usetime=%lu\n",
		lifetime->bytes, lifetime->addtime, lifetime->usetime);
}

void dump_sa_replay_window(struct sa_replay_window *window)
{
	if (!window) {
		printk(KERN_INFO "replay_window is null\n");
		return;
	}

	printk(KERN_INFO "size=%u, seq_num=%u, last_seq=%u, overflow=%u\n",
		window->size, window->seq_num, window->last_seq, window->overflow);
#if 0
	printk(KERN_INFO "bitmap=%x\n", window->bitmap);
#endif
}

void dump_sa_auth_algo(struct sa_algo_info *auth_algo)
{
	if (!auth_algo) {
		printk(KERN_INFO "auth_algo is null\n");
		return;
	}
	printk(KERN_INFO "algo=%s, key_len=%u\n",
		authtoa(auth_algo->algo), auth_algo->key_len);
}

void dump_sa_esp_algo(struct sa_algo_info *esp_algo)
{
	if (!esp_algo) {
		printk(KERN_INFO "esp_algo is null\n");
		return;
	}

	printk(KERN_INFO "algo=%s, key_len=%u, \n",
		esptoa(esp_algo->algo), esp_algo->key_len);
}

void dump_ipsec_sa(struct ipsec_sa *sa)
{
	char buf[BUFSIZE];

	if (!sa) {
		printk(KERN_INFO "dump_ipsec_sa = null\n");
		return;
	}

	memset(buf, 0, BUFSIZE);
	sockaddrtoa((struct sockaddr*)&(sa->src), buf, BUFSIZE);
	printk(KERN_INFO "sadb:src=%s\n", buf);
	memset(buf, 0, BUFSIZE);
	sockaddrtoa((struct sockaddr*)&(sa->dst), buf, BUFSIZE);
	printk(KERN_INFO "sadb:dst=%s\n", buf);
	memset(buf, 0, BUFSIZE);
	sockaddrtoa((struct sockaddr*)&(sa->pxy), buf, BUFSIZE);
	printk(KERN_INFO "sadb:pxy=%s\n", buf);

	printk(KERN_INFO "sadb:proto=%u\n", sa->proto);
	printk(KERN_INFO "sadb:satype=%u\n", sa->ipsec_proto);
	printk(KERN_INFO "sadb:spi=0x%X\n", ntohl(sa->spi));
	printk(KERN_INFO "sadb:repley window:");dump_sa_replay_window(&(sa->replay_window));
	printk(KERN_INFO "sadb:lifetime soft   :");dump_sa_lifetime(&(sa->lifetime_s));
	printk(KERN_INFO "sadb:lifetime hard   :");dump_sa_lifetime(&(sa->lifetime_h));
	printk(KERN_INFO "sadb:lifetime current:");dump_sa_lifetime(&(sa->lifetime_c));

	if (sa->ipsec_proto == SADB_SATYPE_AH) {
		printk(KERN_INFO "sadb:auth:");dump_sa_auth_algo(&(sa->auth_algo));
	}

	if (sa->ipsec_proto == SADB_SATYPE_ESP) {
		printk(KERN_INFO "sadb:esp:");dump_sa_esp_algo(&(sa->esp_algo));
	}

}

